/**
  ******************************************************************************
  * @file    DALI_Stack.c 
  * @author  Pascual Martinez Perez
  * @version V1.0
  * @date    12-Mayo-2016
  * @brief   
  ******************************************************************************
  *  
  * Implementacion de la capa Stack de DALI
  *
  ******************************************************************************
  */


/* Includes ------------------------------------------------------------------*/

#include "DALI.h"
#include "DALI_Config.h"
#include "stm32f4_discovery.h"
#include "stm32f4xx.h"
#include "UtilidadesPlaca.h"
#include "DatosComunes.h"
#include "stm32f4xx_it.h"
#include "DALI_Comandos.h"




/* Private typedef -----------------------------------------------------------*/

/* Private define ------------------------------------------------------------*/


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static unsigned char send_position; // Posicion de los datos a trensferir
static unsigned char send_active; // True si TX activo
static unsigned char send_value; // EL valor del bit a transmitir

static unsigned char rec_position; // Posicion de recepcion de bit
static unsigned char rec_active; // True si la recepcion ha comenzado
static uint32_t rec_value; // mantiene el bit recibido
static unsigned char rec_bit; // numero de bits recibidos
static unsigned char rec_start;  //inicio recepcion;
static unsigned char temp_value;  //valor temporal a transmitir

static unsigned char contadorComandos;  //contador de comandos para pruebas
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
 
void Inicio_Dali()
{
  contadorComandos=0;
  send_active=FALSE;
  rec_active=FALSE;
  
  /* Configure USER Button Interrupcion PA0*/
  BSP_PB_Init(BUTTON_KEY, BUTTON_MODE_EXTI);
  
 
  //Habilito clks del puerto a usar
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  
  
   GPIO_InitTypeDef GPIO_InitStructure; 
  //Pin out DALI  PD1
  /* Pull-up Vdd pin for data output */
  GPIO_InitStructure.Pin =  OUT_DALI_PIN ;
  GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(OUT_DALI_PORT, &GPIO_InitStructure);
    
   //Mantener el bus a 1  OBLIGATORIO
  //HAL_GPIO_WritePin( OUT_DALI_PORT, OUT_DALI_PIN, GPIO_PIN_SET);
  busReposo();
  
  //PIN IN DALI PD3
  GPIO_InitStructure.Pin =  IN_DALI_PIN;
  GPIO_InitStructure.Mode = GPIO_MODE_IT_FALLING;//GPIO_MODE_IT_RISING_FALLING;//  //Flanco bajada
  //GPIO_InitStructure.Speed = GPIO_SPEED_FREQ_HIGH;
  GPIO_InitStructure.Pull= GPIO_NOPULL;
  HAL_GPIO_Init(IN_DALI_PORT, &GPIO_InitStructure);
  
  
  /* Enable and set Button EXTI Interrupt to the lowest priority */
  
  HAL_NVIC_SetPriority(EXTI0_IRQn, 3, 0);
  HAL_NVIC_EnableIRQ(EXTI0_IRQn);
  
  HAL_NVIC_SetPriority(EXTI3_IRQn, 3, 0);
  HAL_NVIC_DisableIRQ(EXTI3_IRQn);
  
  
}//fin inicio DALI 


void TX_Datos_Dali(void)
{
       
    //Comandos a eviar
    //broadcast OFF
    //direccion=DIRECT_ARC_POWER;
    //comando=0xDC;  //Mitad
    //comando=OFF;  //Apagado
    //comando=0xFE;  //TOPE
    //comando=0xAB;  
    
    direccion=0xFF;
    comando=QUERY_ACTUAL_LEVEL;
    
    //inicializa variables
    send_position=0;
    send_value=0;
    send_active=TRUE;
    /*if (contadorComandos % 2==0 )
      
      comando=STEP_DOWN;
    else
      comando=STEP_DOWN;
    contadorComandos++;*/
    
    
}

void RX_Datos_Dali(void)
{
 
  /*rec_position=0;
  rec_active=TRUE;
  rec_bit=0;
  rec_value=0x00;
  respuesta=0x00;*/
  
   
  rec_start=TRUE;
  //rec_int=0;
      
}



/**
  * @brief  EXTI line detection callbacks.
  * @param  GPIO_Pin: Specifies the pins connected EXTI line
  * @retval None
  */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
   
  //Envio Comando
  if (KEY_BUTTON_PIN == GPIO_Pin)
  {
   
    //Si es el pin del pulsador  
    flag |=SEND_DATA; 
}
//Recibo comando
if (IN_DALI_PIN == GPIO_Pin)
{
    
      flag |=ANSWER_EVENT;
      
}


}// Fin HAL_GPIO_EXTI_Callback

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
  
  //Escuchar respuesta
  
  
  if (rec_active==TRUE)
  { 
    //Time out lo marca la norma cuanto hay que escuchar
    if( rec_start==FALSE && rec_position>=44)
    {
        //Para salir
        rec_active=FALSE;
        rec_start=FALSE;
          rec_position=0;
         
           HAL_NVIC_DisableIRQ(EXTI3_IRQn);
          //habilito la interrupcion del pulsador 
         HAL_NVIC_EnableIRQ(EXTI0_IRQn);
         //deshabilito la interupcion de escuchar
         //no hay nada que escuchar
        
       //Indicar no hay respuesta
       //el numero de bits recibidos es cero
           
    }//Fin rec_start  FALSE  
    
    
     else if (rec_start==TRUE)
    {  
      //leo puerto cada tick en las muestras necesarias calculadas
       
      switch (rec_position)
      {
       //Son posiciones que corresponden con cada tick de respuesta
        case 31:
        case 35:
        case 39:
        case 43:
        case 47:
        case 51:
        case 55:
        case 59: 
          
        {
          //leo el puerto de entrada segun muestra
          //Parpadeo del bit azul veo qeu leo y recibo algo
          rec_bit=HAL_GPIO_ReadPin( IN_DALI_PORT, IN_DALI_PIN);
            if (rec_bit==0)
                led_Azul_OFF();
            if (rec_bit==1)
              led_Azul_ON();
            //Acumulo los bits recibidos
            rec_value = (rec_value << 1) |rec_bit;
          
          break;
        } 
      case 68:
        {
          //fin de recepcion
          led_Azul_OFF();
          rec_start=FALSE;
          rec_active=FALSE;
          rec_position=0;
          HAL_NVIC_DisableIRQ(EXTI3_IRQn);
          HAL_NVIC_EnableIRQ(EXTI0_IRQn);
          break;
        }  
      }//Fin switch  
       
      }//fin rec_start  
      rec_position++;
    
  }//Fin rec_active

  
  //EVIAR COMANDO
  if (send_active==TRUE)
  {
      
      //Esta el bus a 1  
      
      //bit inicio
      if (send_position<=3)
      {
        enviaDato(send_position,1);
      }  
      //direccion y dato
      else if (send_position>=4 && send_position<=67)
      {
          
        if(send_position<=35)
        {
          //direccion 4 35
          temp_value = (direccion>>((35-send_position)/4)) & 0x01;//35 pos maxima 4 muetras por bit

        }
        else
        {
          //comando 36 67
          temp_value = (comando>>((67-send_position)/4)) & 0x01;//67 pos maxima 4 muetras por bit
        } 
        
        enviaDato(send_position,temp_value);
        
      }
      //bit 1 final
      //else if (send_position<=68 && send_position<=71)
      else if (send_position==68)
      {
          busReposo();
      }
      //bit 1 final
      /*else if (send_position<=72 && send_position<=75)
      {
          busReposo();
      }*/  
      //Salir 
      else if (send_position>=76)//76
      {
         
        //Habilitamos escucha por interrupcion e inicializamos
        //Fin comando
        send_active = FALSE;
        
         //Inicializo escucha
         rec_position=0;
        rec_bit=0;
        rec_value=0x00;
        temp_value=0x00;
        respuesta=0x00;
        
        //Activo int ext 3 y activo la escucha
         //Me pongo a escuchar
        rec_active=TRUE;
        rec_start=FALSE;
        
        HAL_NVIC_EnableIRQ(EXTI3_IRQn);
        HAL_NVIC_DisableIRQ(EXTI0_IRQn);
        //HAL_NVIC_ClearPendingIRQ(EXTI3_IRQn);
      }  
      
    
      send_position++; 
   }//Fin envio datos send_active=TRUE
 
}//Fin systick  ***          

/* Esta rutina cambia cambia de estado el dato de salida
* codifica el manchester en funcion de en que posicion estoy de la trama 
* y el dato qeu tengo que enviar */
void enviaDato(unsigned char posicion,unsigned char dato)
{
 //deteccion en que parte del bit estamos
      switch (posicion % 4)
      {
          //primera mitad siempre inverso
          case 0:
          case 1:
          {
            dato=!dato;
            break;
          }
        
          
          //Segunda mitad mismo dato
          case 2:
          case 3:
          {
            dato=dato;
            break;
          } 
         
       }//Fin switch  
    
      if (dato==0)
      {  
        //envio 0  //Bus a 5 voltios
         HAL_GPIO_WritePin( OUT_DALI_PORT, OUT_DALI_PIN, GPIO_PIN_RESET);
         led_Verde_OFF();
         //led_Rojo_OFF();
      }  
      else
      {
        //envio 1//Bus a 20 voltios
        HAL_GPIO_WritePin( OUT_DALI_PORT, OUT_DALI_PIN, GPIO_PIN_SET);
        led_Verde_ON();
        //led_Verde_OFF();
      } 

} //Fin envia datoa

// DEja el bus en estado alto
void busReposo()
{
         //envio 1//Bus a 20 voltios
        HAL_GPIO_WritePin( OUT_DALI_PORT, OUT_DALI_PIN, GPIO_PIN_SET);
}  



